--- interface for scenarios
-- 
-- @file	IScenario.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/18

--- @module IScenario
local IScenario = {}

----------------------------------------------------------------
-- public interface
----------------------------------------------------------------
local scenario = { name="", storage={} }

--- Create new instance of scenario
-- @note  internally devided in two parts
--   *1     initialize internal structures  
--   *2     load actual data from persistant storage
-- @note both stages can have no implementation at the concrete
--       scenario level
function scenario:create()
	self:initialize()
	self:load()
end

--- Called internally by IScenario:create()
-- @note initialize internal tables and constants here 
function scenario:initialize()
end

--- Called internally by IScenario:create()
-- @note here all load state dependent variables should
--       be loaded from external storage
function scenario:load()
end

--- Run main event handler for this scenario
-- @param pkt		actual data packet
function scenario:exec(pkt)
	return false
end

--- Actually it will just deinitialize scenario
--        after ends of one iteration cycle
-- @note  internally devided in two parts
--   *1     store state dependent variables to persistant storage  
--   *2     deinitialize internal data structures
-- @note both stages can have no implementation at the concrete
--       scenario level
function scenario:destroy()
	self:store()
	self:deinitialize()
end

--- Store state dependent variables to persistant storage
function scenario:store()
end

--- Deinitialize internal data structures
function scenario:deinitialize()
end

--- Retreiving scenarion name
function scenario:getName()
	return self.name
end


--- Constructor
function IScenario:new()
	local obj = {}
	setmetatable(obj, {__index = scenario})
	return obj
end

return IScenario

