--- instantiate various types of scenarios
-- 
-- @file	ScenariosFactory.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/18

--- @module ScenariosFactory
local ScenariosFactory = {}

local NAMES = {
	        "startUp"
	       ,"dataSync"
	       ,"modStatus"
	   	   ,"wakeUp"
	  	   ,"roomState"
	       ,"roomEquipment"
	       ,"postingSimple"
        }
local MODULES = {
		        "scrStartUp"
		       ,"scrDataSync"
		       ,"scrModStatus"
		   	   ,"scrWakeUp"
		  	   ,"scrRoomState"
		       ,"scrRoomEquipment"
		       ,"scrPostingSimple"
		      }
local IDENTS = {}

----------------------------------------------------------------
-- public interface
----------------------------------------------------------------
local factory = { storage={} }

--- Create scenario object by ident value
-- @param ident		A identification of the executed scenario
-- @return scenario object 
function factory:instantiate(ident)
	local t		= nil
	local name  = nil
	if "number" == type(ident) then
		t    = require(MODULES[ident])
		name = NAMES[ident]
	elseif "string" == type(ident) then
		t 	 = require(MODULES[IDENTS[ident]])
		name = MODULES[IDENTS[ident]]
	end

	return t and t:new(name, self.storage)
end

--- Create new iterator
-- @param args		An arguments for functor
-- @param functor	A name of the function which will be executed
function factory:iterator(args, functor)
	local i   = 0
	local pos = #NAMES
	if not functor then
		return function()
		    	i = i+1
		      	if i <= pos then
		      		return self:instantiate()
		      	end
	       	end
	else
		return function()
	      		i = i+1
	      		if i <= pos  then
			   		local callee = functor and require(MODULES[i])[functor]
			   		local status = callee and callee(args)
			   		return NAMES[i], status
				end
	   		end
	end
	return false
end


--- Initialize factory
-- @param storage		data storage interface
-- @return factory object
function ScenariosFactory:new(storage)
	-- prepare ident table
    for i=1, #NAMES do
      IDENTS[NAMES[i]] = i
    end

	local obj = {}
	setmetatable(obj, {__index = factory})
	obj.storage = storage
	return obj
end

return ScenariosFactory
