--- implementation of ModuleStatus scenario
-- 
-- @file	scrModStatus.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/18

--- @module scrModStatus
local scrModStatus = {}

----------------------------------------------------------------
-- implementation details
----------------------------------------------------------------
local SYS = require("DataModel").SYS

--- Internal helper for checking type and content of incoming packet
-- @param p			incoming data packet
-- @return true in case "p" is valid exemplar of "DataModel", false otherwise 
local function __validatePacket(p)
	if not p or "table" ~= type(p) then
		return false
	end
	if not p.getIdentification or "DataModel" ~= p:getIdentification() then
		return false
	end
	return true
end


--- Internal helper for checking type and content of incoming packet
-- @param p			incoming data packet
-- @return true in case "p" is SystemMessage of type "M_MODULE_STATUS", false otherwise 
local function __checkSysPacket(p)
	if not p:isSystem() then
		return false 
	end
	local sysType = p:getItem("ident")
	return sysType and SYS.M_MODULE_STATUS == sysType
end

----------------------------------------------------------------
-- public interface
----------------------------------------------------------------
local scenario = require"IScenario":new()        -- meta class

--- @see description in IScenario 
function scenario:initialize()
	self.controller = require("ModStatusController"):getInstance()
end

--- @see description in IScenario
function scenario:exec(pkt)
	if not __validatePacket(pkt) then 
		return false
	end
    
	local mStat    = pkt:getItem("modStatus")
    local mModData = pkt:getRepeatedItemByInternalObj(mStat.mModData)
	if mStat and self.controller then
        self.controller.setData(mStat.mModName, mModData)
        local modName, modStatus = self.controller.setStatus(mStat.mModName, mStat.mModStatus)
		print("Set status %q for iface %q", modStatus or "", modName or "")
	end
end

--- Analize incoming packet and decide "whether 
--  it can be processed by current module"
-- @param pkt		incoming packet for analizing
-- @return boolean operation status
function scrModStatus.staticCheck(pkt)
	if not __validatePacket(pkt) then 
		return false
	end
	return __checkSysPacket(pkt)
end


--- Constructor
function scrModStatus:new(name, storage)
	local obj = {}
	setmetatable(obj, {__index = scenario})
	obj.name    = name
	obj.storage = storage
	return obj
end

return scrModStatus