--- add-on for simplify storage access 
-- 
-- @file	DataProvider.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/21

----------------------------------------------------------------
-- public interface
----------------------------------------------------------------
local DataProvider = {
	_instance  = nil,
}

function DataProvider:getInstance(storage)
    if self._instance then
    	return self._instance
    end
	if not storage then
		return nil
	end

	self._instance  = DataProvider
	local o         = self._instance
    	  o.storage = storage

	----------------------------------------------------------------
	--- Internal helpers
	----------------------------------------------------------------
    local function __setPersistByKey(k, v)
		return o.storage:put("hotel", {ident="sql",val={act="upd",table="defines",fields={"value"},where="name='"..(k or "").."'"}}, {v})
    end
    local function __getPersistByKey(k)
		return o.storage:get("hotel",{ident="sql",val={table="defines",fields={"value"},where="name='"..(k or "").."'"}})
    end
    local function __setTempByKey(k, v)
    	return o.storage:putTemp(k,v)
    end
    local function __getTempByKey(k)
    	return o.storage:getTemp(k)
    end
    local function __delTempByKey(k)
    	return o.storage:delTemp(k)
    end
    local function __setAstByKey(k, v)
    	return o.storage:put("ast", k, v)
    end
    local function __getAstByKey(k)
    	return o.storage:get("ast", k)
    end
    local function __delAstByKey(k)
    	return o.storage:del("ast", k)
    end
    local function __delTreeAstByKey(k)
    	return o.storage:del("ast", {ident="tree", val=k})
    end

	----------------------------------------------------------------
	--- Access to generic information
	----------------------------------------------------------------
    o.setProcPid = function()
    	local pid = require"posix".getpid("pid")
    	return __setPersistByKey("proc_pid", pid)
    end
    o.getProcPid = function()
    	return __getPersistByKey("proc_pid") 
    end
    o.getStartTime = function()
    	return __getPersistByKey("system_started")
    end
    o.getPrvStartTime = function()
    	return __getPersistByKey("prv_system_started")
    end

	----------------------------------------------------------------
	--- Statuses of interfaces
	----------------------------------------------------------------
    o.setInterfaceStatus = function(ifn,v)
 		return __setTempByKey("IFACE_STATUS/"..(ifn or ""),v)
    end
    o.getInterfaceStatus = function(ifn)
       	return __getTempByKey("IFACE_STATUS/"..(ifn or ""))
    end
    o.delInterfaceStatus = function(ifn)
       	return __delTempByKey("IFACE_STATUS/"..(ifn or ""))
    end
    o.setInterfaceData = function(ifn,d)
        if "table" == type(d) and 0 ~= #d then
            local rv = {}
            for k,v in pairs(d) do  -- remove index value to decrease size
                if "number" ~= type(k) then
                    rv[k] = v
                end
            end
            rv = require"dkjson".encode(rv)
            return __setTempByKey("IFACE_DATA/"..(ifn or ""),rv)
        end
    end
    o.getInterfaceData = function(ifn,item)
        local d = __getTempByKey("IFACE_DATA/"..(ifn or "")) or ""
              d = require"dkjson".decode(d) or {}
        if item then
            return d[item]
        end
        return d
    end
    o.delInterfaceData = function(ifn)
        return __delTempByKey("IFACE_DATA/"..(ifn or ""))
    end

	----------------------------------------------------------------
	--- In-memory dividers
	----------------------------------------------------------------
    o.setDivider = function(name,v)
 		return __setTempByKey("DIVIDER/"..(name or ""),v)
    end
    o.getDivider = function(name)
       	return __getTempByKey("DIVIDER/"..(name or ""))
    end
    o.delDivider = function(name)
       	return __delTempByKey("DIVIDER/"..(name or ""))
    end

	----------------------------------------------------------------
	--- Access to Hotel configuration
	----------------------------------------------------------------
    o.setHotelConfig = function(cfg)
    	for f, v in cfg:ListFields() do
    		local k = "table" == type(f) and f.name and f.name
    		if k then
        		if "allocated" ~= k and "isInitial" ~= k and "MemCachePort" ~= k 
        		   and "MemCacheHost" ~= k and "astDbFamily" ~= k then
        			__setTempByKey("HOTEL-CONFIG/"..k, v)
        		end
        	end
    	end
    end
    o.getWakeUpContext = function()
       	return __getTempByKey("HOTEL-CONFIG/WakeUpContext") or "users"
    end
    o.getWakeUpTries = function()
       	return __getTempByKey("HOTEL-CONFIG/WakeUpTries")
    end
    o.getWakeUpTimeout = function()
       	return __getTempByKey("HOTEL-CONFIG/WakeUpTimeout")
    end
    o.getWakeUpExten = function()
       	return __getTempByKey("HOTEL-CONFIG/WakeUpExten")
    end
    o.getBillingType = function()
    	local val = __getTempByKey("HOTEL-CONFIG/BillingType")
    	return tonumber(val) or 0
    end
    o.getWsMappingFile = function()
       	return __getTempByKey("HOTEL-CONFIG/wsMapping")
    end
    o.getHsprjFile = function()
       	return __getTempByKey("HOTEL-CONFIG/hotelConfig")
    end
    o.getReSyncLevel = function()
       	return tonumber(__getTempByKey("HOTEL-CONFIG/ReSyncLevel"))
    end
    o.getMinibarType = function()
    	local val = __getTempByKey("HOTEL-CONFIG/MinibarType")
    	return tonumber(val) or 0
    end
    o.getDNDBehaviour = function()
    	local val = __getTempByKey("HOTEL-CONFIG/DnDBehaviour")
    	return tonumber(val) or 0
    end
    o.getEnableAutomation = function()
    	local val = __getTempByKey("HOTEL-CONFIG/EnableAutomation")
    	return tonumber(val) or 0
    end
    o.getRoomStateNotification = function()
        local val = __getTempByKey("HOTEL-CONFIG/BadgesController")
        return 0 == tonumber(val) and 1 or 0
    end
    o.getBadgesResponseTimeout = function()
        local val = __getTempByKey("HOTEL-CONFIG/BadgesTimeout")
        val       = require"timerHelper".fromTimeCounter(tonumber(val) or 20)  -- seconds in system ticks
        return val
    end
    
    o.delWakeUpContext = function()
       	return __delTempByKey("HOTEL-CONFIG/WakeUpContext")
    end
    o.delWakeUpTries = function()
       	return __delTempByKey("HOTEL-CONFIG/WakeUpTries")
    end
    o.delWakeUpTimeout = function()
       	return __delTempByKey("HOTEL-CONFIG/WakeUpTimeout")
    end
    o.delWakeUpExten = function()
       	return __delTempByKey("HOTEL-CONFIG/WakeUpExten")
    end
    o.delBillingType = function()
       	return __delTempByKey("HOTEL-CONFIG/BillingType")
    end
    o.delWsMappingFile = function()
       	return __delTempByKey("HOTEL-CONFIG/wsMapping")
    end
    o.delHsprjFile = function()
       	return __delTempByKey("HOTEL-CONFIG/hotelConfig")
    end
    o.delReSyncLevel = function()
       	return __delTempByKey("HOTEL-CONFIG/ReSyncLevel")
    end
    o.delMinibarType = function()
       	return __delTempByKey("HOTEL-CONFIG/MinibarType")
    end
    o.delDNDBehaviour = function()
       	return __delTempByKey("HOTEL-CONFIG/DnDBehaviour")
    end
    o.delEnableAutomation = function()
    	return __delTempByKey("HOTEL-CONFIG/EnableAutomation")
    end
    o.delRoomStateNotification = function()
        return __delTempByKey("HOTEL-CONFIG/BadgesController")
    end
    o.delBadgesResponceTimeout = function()
        return __delTempByKey("HOTEL-CONFIG/BadgesTimeout")
    end

	----------------------------------------------------------------
	--- Wakeup's data processing
	----------------------------------------------------------------
    o.getWakeUpMap = function(rid)
    	return __getTempByKey("ROOM/"..(rid or "000").."/WAKE-UP-MAP")
    end
    o.getAstWakeUpMap = function(rid)
    	return __getAstByKey((rid or "000").."/WAKE-UP-MAP")
    end
    o.setWakeUpMap = function(rid, val, onlyCache)
    	if val then
        	__setTempByKey("ROOM/"..(rid or "000").."/WAKE-UP-MAP", val)
        	if not onlyCache then
        		__setAstByKey((rid or "000").."/WAKE-UP-MAP", val)
        	end
        end
    end
    o.delWakeUpMap = function(rid, onlyCache)
    	__delTempByKey("ROOM/"..(rid or "000").."/WAKE-UP-MAP")
    	if not onlyCache then
    		__delAstByKey((rid or "000").."/WAKE-UP-MAP")
    	end
    end
    
    o.setWakeUpData = function(rid, ident, val, onlyCache)
    	if ident and val then
        	__setTempByKey("ROOM/"..(rid or "000").."/WAKE-UP/"..ident, val)
        	__setTempByKey("ROOM/"..(rid or "000").."/WAKE-UP/"..val, ident)
        	if not onlyCache then
    			__setAstByKey((rid or "000").."/WAKE-UP/"..(ident or ""), val)
    		end
        end
    end
    o.delWakeUpData = function(rid, ident, timeVal, onlyCache)
    	if ident and timeVal then
    		__delTempByKey("ROOM/"..(rid or "000").."/WAKE-UP/"..(timeVal or ""))
    		__delTempByKey("ROOM/"..(rid or "000").."/WAKE-UP/"..(ident or ""))
    		if not onlyCache then
    			__delAstByKey((rid or "000").."/WAKE-UP/"..(ident or ""))
    		end
    	end	
    end
    o.getWakeUpData = function(rid, ident)
    	if ident then
    		return __getTempByKey("ROOM/"..(rid or "000").."/WAKE-UP/"..ident)
    	end
    end
    o.getWakeUpDataByTimeVal = function(rid, timeVal)
    	if timeVal then
    		return __getTempByKey("ROOM/"..(rid or "000").."/WAKE-UP/"..(timeVal or ""))
    	end
    end
    o.getAstWakeUpData = function(rid, ident)
    	if ident then
    		return __getAstByKey((rid or "000").."/WAKE-UP/"..(ident or ""))
    	end
    end
    o.getAllwakeUp = function(rid)
    	if not rid then
    		return false
    	end
    	
    	local isFound = false
    	local wkItems = {}
    	local wkMap   = o.getWakeUpMap(rid) or ""
    	for wkId in wkMap:gmatch("(%d+)!?") do
    		isFound = true
    		wkItems[wkId] = o.getWakeUpData(rid, wkId)
    	end
    	
    	if isFound then
    		return wkItems, wkMap
    	end
    	return false
    end
    o.setWakeUpSyncFlag = function()
    	__setTempByKey("FLAGS/SYNC-WK", 1)
    end
    o.getWakeUpSyncFlag = function()
    	return __getTempByKey("FLAGS/SYNC-WK")
    end
    o.delWakeUpSyncFlag = function()
    	__delTempByKey("FLAGS/SYNC-WK")
    end
    
    ----------------------------------------------------------------
	--- Guest data processing (Only for room oriented configuration)
	----------------------------------------------------------------
	local guestTable = "guests"
	o.setGuestData = function(gData, roomInfo, gid)
		if "table" ~= type(roomInfo) or "table" ~= type(gData) then
			return nil
		end
		
		local rid 	 = roomInfo.ident
		local rAptId = roomInfo.aptid 
		if not rid or not rAptId then
			return nil
		end
		
		-- processing input values
    	local strConv 	= require"strConverter"
    	local dateConv	= require"dateConverter"
    	
    	local data = {
    		 name 		= strConv.toHex(gData.name) or ""
    		,surname 	= strConv.toHex(gData.surname) or ""
    		,lang		= strConv.toHex(gData.lang) or ""
    		,arrival	= dateConv.fias2unixUTC(gData.arrDate, gData.time) or 0 
    		,departure 	= dateConv.fias2unixUTC(gData.depDate) or 0
    		,gtype		= tonumber(gData.gtype) or 0
    		,rid		= rid or ""
    	}
		
		if gid then		-- it is update request
			local fld_t = {}
			local val_t = {}
			for key, val in pairs(data) do
				if key and val and "" ~= val and 0 ~= val then
					table.insert(fld_t, key)
					table.insert(val_t, val)
				end
			end
			if 0 ~= #fld_t then
				o.storage:put("hotel", {ident="sql",val={act="upd",table=guestTable,fields=fld_t,where="id="..gid}}, val_t)
			end
		else      	-- create new record to in hotel db
			local d = data
			
			o.storage:put("hotel", {ident="sql",val={act="new",table=guestTable, 
					    fields={"name", "surname", "lang", "arrival", "departure", "gtype", "rid"}}}, {d.name, d.surname, d.lang, d.arrival, d.departure, d.gtype, d.rid})
			
			gid = o.storage:get("hotel", {ident="sql",val={table=guestTable,fields={"id"},where="rid='"..rid.."'",postfix="ORDER by id DESC LIMIT 1"}})
		end
		
		return gid
	end
	o.getGuestData = function(searchList)
		if "table" ~= type(searchList) then
			return {}
		end
		
		local whereQueryTable = {}
		for name, val in pairs(searchList) do
			if name and val and "" ~= val then
				whereQueryTable[#whereQueryTable + 1] = string.format("%s='%s'", name, val)
			end
		end
		if 0 == #whereQueryTable then
			return {}
		end
		local whereQuery = table.concat(whereQueryTable, " OR ")
		
		local fields = {"id", "name","surname","lang", "arrival", "departure", "gtype", "rid"}
		local rt = o.storage:get("hotel", {ident="sql",val={table=guestTable,fields=fields,where=whereQuery,postfix="ORDER by id DESC LIMIT 1"}})
		
		local gid, name, surname, lang, arrDate, depDate, gtype, rid = unpack(rt or {}, 1, #fields)
		
		-- processing output values
		local strConv = require"strConverter"
		
		gid  	= tonumber(gid) or 0
		name 	= strConv.fromHex(name) or ""
    	surname = strConv.fromHex(surname) or ""
    	lang	= strConv.fromHex(lang) or ""
    	arrDate = tonumber(arrDate) or 0
    	depDate = tonumber(depDate) or 0
    	gtype   = tonumber(gtype) or 0
    	rid		= rid or ""
		
		return {
			 gid	 = 0 ~= gid and gid
			,name 	 = "" ~= name and name
    		,surname = "" ~= surname and surname
    		,lang	 = "" ~= lang and lang
    		,gtype	 = 0 ~= gtype and gtype
    		,arrDate = 0 ~= arrDate and arrDate
    		,depDate = 0 ~= depDate and depDate
    		,rid     = "" ~= rid and rid
		}
	end
	o.getGuestDataById = function(gid)
		return o.getGuestData({id = gid})
	end
	o.getGuestDataByRoom = function(rid)
		return o.getGuestData({rid = rid})
	end
	o.delGuestData = function(gid)
		return o.storage:del("hotel", {ident="sql",val={table=guestTable,where="id="..(gid or "")}})
	end
	
	----------------------------------------------------------------
	--- Key/badge data processing
	----------------------------------------------------------------
	o.setKeyCacheMapVal = function(val)
		local map = __getTempByKey("CACHE/KEY-MAP/") or ""
		__setTempByKey("CACHE/KEY-MAP/", map..val.."|")
	end
	
	o.delKeyCacheMapVal = function(val)
		local map = __getTempByKey("CACHE/KEY-MAP/") or ""
		if "" ~= map then
			if val and "" ~= val then
				map = map:gsub(val.."|", "")
				if map and "" ~= map then
    				__setTempByKey("CACHE/KEY-MAP/", map)
    			else
    				__delTempByKey("CACHE/KEY-MAP/")
    			end
    		end
		end
	end
	
	o.iterateKeyCacheMap = function()
		local map = __getTempByKey("CACHE/KEY-MAP/") or ""
		return map:gmatch("([^|]+)|")
	end
	
	o.setKeyCache = function(cachedUID, data, withoutMap)
		if cachedUID and "table" == type(data) then
			local val = string.format("%s|%d|%s!%s|%s!%d|%s",
				 data.reqType, data.divider or 0			-- internal system fields
				,data.keyCoder or "", data.ws or ""			-- mandatory fields
				,data.rid or "", data.badgeType or 0     -- check-in fields
				,data.badgeCnt or ""						-- check-out fields
			)
			__setTempByKey("CACHE/KEY/"..cachedUID, val)
			if not withoutMap then
				o.setKeyCacheMapVal(cachedUID)
			end
		end
	end
	
	o.setKeyCacheValueOnly = function(cachedUID, data)
		return o.setKeyCache(cachedUID, data, true)
	end
	
	o.checkKeyCache = function(cacheUID)
		return 	__getTempByKey("CACHE/KEY/"..(cacheUID or ""))	
	end
	
	o.getKeyCache = function(cachedUID)
		local rv = nil
		if not cachedUID then
			return rv
		end
		
		local val = __getTempByKey("CACHE/KEY/"..cachedUID)
		if val then
			local rt, div, kc, ws, rid, bt, cnt = string.match(val, "^(%w+)|(%d+)|(.*)!(.*)|(%w+)!(%d+)|(.*)$")
   			rv = {
   				 divider     = tonumber(div)
   				,reqType     = rt
   				,keyCoder    = "" ~= kc and kc
   				,ws  	     = "" ~= ws and ws
   				,rid         = "" ~= rid and rid 
   				,badgeType   = bt
   				,badgeCnt    = tonumber(cnt) or 0
   			}
		end
		return rv 
	end
	
	o.delKeyCache = function(cachedUID)
		if cachedUID then
			o.delKeyCacheMapVal(cachedUID)
			__delTempByKey("CACHE/KEY/"..cachedUID)
		end
	end
	
	
	local badgeTable = "badges"
	o.setBadge = function(bData, serial)
		if "table" == type(bData) then
			if serial then	-- it is update request
				local fld_t = {}
    			local val_t = {}
    			for key, val in pairs(bData) do
    				if key and val and "" ~= val then
    					table.insert(fld_t, key)
    					table.insert(val_t, val)
    				end
    			end
    			
				o.storage:put("hotel", {ident="sql", val={act="upd",table=badgeTable,fields=fld_t,where="serial='"..serial.."'"}}, val_t)
			else	-- create new record
				local d = bData
				o.storage:put("hotel", {ident="sql",val={act="new",table=badgeTable, 
					    fields={"serial", "startv", "endv", "type", "common", "suite", "rid"}}}, {d.serial, d.startv or 0, d.endv or 0, d.type or 0, d.common or "", d.suite or "", d.rid or ""})
				
			end
		end
	end
	
	o.getBadge = function(searchList, isTableResult)
		local rv = nil
		if "table" ~= type(searchList) then
			return rv
		end
		
		local where = {}
		for name, val in pairs(searchList) do
			if name and val then
				local ptrn = string.format("%s='%s'", name, val)
				table.insert(where, ptrn)
			end
		end
		if 0 == #where then
			return rv
		end
		
		local fields   = {"id", "serial", "startv", "endv", "type", "common", "suite", "rid"}
		local data     = o.storage:get("hotel", {ident="sql",val={table=badgeTable,fields=fields,where=table.concat(where, " OR ")}})
		local iterator = nil 
		if "table" == type(data) then
			local idx = 0
			iterator = function()
				idx = idx + 1
				if 1 >= idx then
					return unpack(data, 1, #fields)
				end
			end
		elseif "function" == type(data) then
			iterator = data
		end
		if not iterator then
			return rv
		end
		
		rv = {}
		for id, serial, startv, endv, type, common, suite, rid in iterator do
			local badgeInfo = {
    				 id 	= id
    				,serial = serial
    				,startV = startv
    				,endV	= endv
    				,type	= tonumber(type)
    				,common = common
    				,suite  = suite
    				,rid	= "" ~= rid and rid
    			}
    		if not isTableResult then
    			rv = badgeInfo
    			break
    		else
    			rv[#rv + 1] = badgeInfo
    		end
		end
		
		return rv
	end
	
	o.getBadgeBySn = function(sn)
		return o.getBadge({serial = sn})
	end
	
	o.getBadgeById = function(id)
		return o.getBadge({id = id})
	end
	
	o.getBadgeByRoomIdent = function(rid)
		return o.getBadge({rid = rid}, true)
	end
	
	o.delBadge = function(snList)
		if snList then
    		if "table" ~= type(snList) then
    			snList = {snList}
    		end
    		local where = "serial='" .. table.concat(snList, "' OR serial='") .. "'"
    		o.storage:del("hotel", {ident="sql",val={table=badgeTable,where=where}})
    	end
	end
	
	----------------------------------------------------------------
	--- Post Simple cache processing
	----------------------------------------------------------------
	o.setCachePS = function(id, rn, state)
		if id and rn then
			local val = string.format("%d!%s!%s", id, rn, state or "")
			__setTempByKey("CACHE/PS", val)
		end
	end
	
	o.getCachePS = function()
		local rv  			= nil
		local val 			= __getTempByKey("CACHE/PS")
		if val then
    		local id, rn, state = string.match(val, "(%d+)!(.*)!(%w+)")
    		if id and rn and state then
    			rv = {
        			 id 	= id
        			,rn 	= rn
        			,state 	= state
    			}
    		end
    	end
		return rv
	end 
	
	o.delCachePS = function()
		return __delTempByKey("CACHE/PS")
	end
	
	o.setCachePSCounter = function(cnt, id)
		if cnt and id then
			local val = string.format("%d!%d", cnt, id)
			__setTempByKey("CACHE/PS-CNT", val)
		end
	end
	
	o.getCachePSCounter = function()
		local rv 	  = nil
		local val 	  = __getTempByKey("CACHE/PS-CNT")
		if val then
    		local cnt, id = string.match(val, "(%d+)!(%d+)")
    		if cnt and id then
    			rv = {
        			 cnt = tonumber(cnt)
        			,id	 = id
        		}
        	end
        end
    	return rv
	end
	
	o.delCachePSCounter = function()
		return __delTempByKey("CACHE/PS-CNT")
	end
	
	----------------------------------------------------------------
	--- In-memory synchronization flags
	----------------------------------------------------------------
    o.getInitializationFlag = function()
    	return __getPersistByKey("just_created") 
    end
    o.setInitializationFlag = function(v)
    	if tonumber(v) then
    		__setPersistByKey("just_created", v)
    	end
    end
    
     o.setMagicNumber = function()
    	__setTempByKey("MAGIC-NUMBER", "0xa580c12b")
    end
    o.getMagicNumber = function()
    	return __getTempByKey("MAGIC-NUMBER")
    end
    
    o.setSyncState = function(v)
    	if v then
    		__setTempByKey("FLAGS/SYNC-STATE", v)
    	end
    end
    o.getSyncState = function(v)
    	return __getTempByKey("FLAGS/SYNC-STATE")
    end
    o.delSyncState = function()
    	__delTempByKey("FLAGS/SYNC-STATE")
    end

    return o
end

return DataProvider

