--- FIFO for GuestData requests by using Memcached storage
-- 
-- @file	GuestDataQueue.lua 
-- @author	Vyacheslav Khudan
-- @created 2015/06/09

--- @module GuestDataQueue
local GuestDataQueue = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------
--- Set data from table to memcached value
local function __set_guest_data(pkt)
	if not pkt or "table" ~= type(pkt) then
		return false
	end
	if not pkt.getIdentification or "DataModel" ~= pkt:getIdentification() then
		return false
	end
	
	local data, err = pkt:serialize()
	if not data then
		return false, string.format("Packet data serialization error: %s", err)
	end
	
	data = require"strConverter".toB64(data)
	
	return string.format("%s", data)
end

--- Get data from memcached value to table
local function __get_guest_data(data)
	if not data or "" == data or "table" == type(data) then
		return false
	end
	
	data = require"strConverter".fromB64(data)
	
	local pkt, err = require"DataModel":new(data)
    if not pkt then
    	return false, string.format("Couldn't parse incoming packet: %s", err)
    end	
    return pkt
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local queue = require"memcQueue":new("GDQueue", __set_guest_data, __get_guest_data) -- meta class

function GuestDataQueue:new()
	local obj = {}
	setmetatable(obj, {__index = queue})
	
	return obj
end

return GuestDataQueue
