--- module status controller
-- 
-- @file	ModStatusController.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/19

----------------------------------------------------------------
-- public interface
----------------------------------------------------------------
local ModStatusController = {
	_instance    = nil,
	STRINGS      = {"offline","online","notInUse"},
}

function ModStatusController:getInstance(provider)
    if self._instance then return self._instance end
	if not provider then return nil end

	self._instance = ModStatusController
	local o = self._instance

	o.provider  = provider
	o.STATUSES  = {
			 	  	   ["1"] = o.STRINGS[1]
					  ,["0"] = o.STRINGS[2]
					  ,["-1"]= o.STRINGS[3]
					  ,[o.STRINGS[1]] =  1
					  ,[o.STRINGS[2]] =  0
					  ,[o.STRINGS[3]] = -1
			 	  }
	o.DEFLT_STATUS = o.STRINGS[3]

    o.getStatus = function(mod)
    	if not o.provider then
    		return o.DEFLT_STATUS 
    	end 
    	local stat = o.provider.getInterfaceStatus(mod)
    	return stat and o.STATUSES[tostring(stat)] or o.DEFLT_STATUS
    end
    
    o.getStatus4Sync = function(mod)
        local rv = o.DEFLT_STATUS
        if not o.provider then
            return rv
        end
        
        repeat
            if "FIAS" == mod and
              not o.getData(mod, "PB") and 
              "0" == o.provider.getInterfaceStatus("FIAS") then 
                break
            end
            if "XOPEN" == mod then
                if 1 == o.provider.getRoomStateNotification() or 
                   1 ~= o.provider.getEnableAutomation() then
                    break
                end
            end
            
            rv = o.provider.getInterfaceStatus(mod)
            rv = o.STATUSES[tostring(rv)]
        until true
        
        return rv or o.DEFLT_STATUS
    end
    
    o.setStatus = function(mod, status)
    	if not mod or not status then
    		return false
    	end

		local mod	= mod:upper():match("^(.*)CONTROLLER$") or mod
    	local value = "number" == type(status) and status or o.STATUSES[status]
    	      value = value or o.STATUSES[o.DEFLT_STATUS]
        if o.provider and mod then
            o.provider.setInterfaceStatus(mod, value)
            return mod, o.STATUSES[tostring(value)]
        end
        return false
    end
    
    o.setData = function(mod, data)
        if not mod or "table" ~= type(data) then
            return false
        end
        
        local mod = mod:upper():match("^(.*)CONTROLLER$") or mod
        if o.provider then
            o.provider.setInterfaceData(mod, data)
            return true
        end
        return false
    end
    
    o.getData = function(mod, item)
        return o.provider.getInterfaceData(mod, item)
    end

    o.isOnline = function(mod)
        return o.getStatus(mod) == "online"
    end
    
    

    return o
end

return ModStatusController

