--- FIFO for PS records by using SQLite storage
-- 
-- @file	PostingQueue.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/23
-- 
-- base idea comes from   http://flask.pocoo.org/snippets/88/

--- @module PostingQueue
local PostingQueue = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local queue = {		        -- meta class
	target = "ps_queue"
}

function queue:size()
	local rv = 0
	
	if self.raw.connect() then
		local cursor = self.raw.exec("SELECT COUNT(*) FROM "..self.target)
		rv = tonumber(self.raw.steps(cursor)) or 0
		self.raw.disconnect()
	end
	return rv
end

--- Pushes a value at the tail of the heap
-- @param ps			posting record to insert into
function queue:push(ps)
	if not ps then 
		return false
	end
	
	if not ps.roomNumber or not ps.type then
		return false
	end
	
--[[
         	 id			INTEGER	PRIMARY KEY AUTOINCREMENT
        	,room		TEXT		NOT NULL
        	,type		TEXT		NOT NULL
        	,date		TEXT		NOT NULL
        	,time		TEXT		NOT NULL
        	,dst		TEXT		
        	,duration 	TEXT		
        	,cost		TEXT
        	,pulse		INTEGER		DEFAULT 0
        	,article	INTEGER		DEFAULT 0
        	,items		INTEGER		DEFAULT 0
--]]
	local vals = {
		 room		= ps["roomNumber"]
		,type		= ps["type"] or ""
		,date		= ps["date"] or ""
		,time		= ps["time"] or ""
		,dst		= ps["dst"] or ""
		,duration   = ps["duration"] or ""
		,cost		= ps["cost"] or ""
		,pulse		= tonumber(ps["pulse"]) or 0
		,article	= tonumber(ps["article"]) or 0
		,items		= tonumber(ps["items"]) or 0
	}
	
	-- converting val table
	for _, name in pairs(self.fields) do
		if "string" == type(vals[name]) then
			vals[name] = "'"..vals[name].."'"
		end
		table.insert(vals, vals[name])
	end
	
	local rs = nil
	if self.raw.connect() then
		local query = string.format("INSERT INTO %s(%s) VALUES(%s)", self.target, table.concat(self.fields,','), table.concat(vals,','))
		rc = self.raw.exec(query)
		self.raw.disconnect()
	end 
	return rs 
end

--- Return the PostingRecord at the head of the queue
-- @return item from the head of queue
-- @note item will be removed from queue
function queue:pop()
	local rv = self:top()
	if rv and rv.id then
		if self.raw.connect() then
    		local query = string.format("DELETE FROM %s WHERE id = %d", self.target, rv.id)
    		self.raw.exec(query)
    		self.raw.disconnect()
    	end
	end
	
	return rv
end

--- Return the PostingRecord at the head of the queue
-- @return item from the head of queue
function queue:top()
	local rv     = nil
	local query  = string.format("SELECT id,%s FROM %s ORDER BY id LIMIT 1", table.concat(self.fields,','), self.target)
	
	if self.raw.connect() then
    	local cursor = self.raw.exec(query)
    	if cursor then
    		local res = self.raw.steps(cursor)
    		if "table" == type(res) then
    			rv  = {
    				 id			= tonumber(res["id"])
    				,roomNumber	= res["room"]
    				,type		= res["type"] or ""
    				,date		= res["date"] or ""
    				,time		= res["time"] or ""
    				,dst		= res["dst"] or ""
        			,duration   = res["duration"] or ""
        			,cost		= res["cost"] or ""
        			,pulse		= tonumber(res["pulse"]) or 0
        			,article	= tonumber(res["article"]) or 0
        			,items		= tonumber(res["items"]) or 0
    			}
    			
    			-- fixup Post sequense number
    			local psIdLimit = 99990
    			if psIdLimit <= rv.id then
	    			self.raw.exec("DELETE FROM SQLITE_SEQUENCE WHERE name='"..self.target.."'")
	    		end
    		end
    	end
    	self.raw.disconnect()
    end
	return rv
end

--- Clear the queue
function queue:clear()
	if self.raw.connect() then
		self.raw.exec("DELETE FROM "..self.target)
		self.raw.disconnect()
	end
end

--- Checks if the queue is empty
function queue:isEmpty()
	return 0 == self:size()
end


function PostingQueue:new()
	local obj = {}
	setmetatable(obj, {__index = queue})

	obj.fields           = {"room","type","date","time","dst","duration","cost","pulse","article","items"}
    obj.raw              = require("sqliteRawInterface"):getInstance()

	return obj
end

return PostingQueue
