--- FIFO for WakeUp requests by using Memcached storage
-- 
-- @file	WakeUpQueue.lua 
-- @author	Vyacheslav Khudan
-- @created 2015/05/25

--- @module WakeUpQueue
local WakeUpQueue = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------
--- Set data from table to memcached value
local function __set_wk_data(data)
	if "table" ~= type(data) then
		return false
	end
	
	local ifn  = data.ifn
	local treq = data.treq
	local rn   = data.rn or ""
	if not ifn or not treq or "" == rn then
		return false
	end
	
	local da   = data.da or ""
	local ti   = data.ti or ""
	
	local wkId = data.wkId or ""
	
	return string.format("%s!%s!%s!%s!%s!%s", ifn, treq:upper(), rn, da, ti, wkId)
end

--- Get data from memcached value to table
local function __get_wk_data(data)
	local rv = false
	if not data or "" == data or "table" == type(data) then
		return rv
	end
	
	local ifn, treq, rn, da, ti, wkId = data:match("^(%w+)!(%w+)!(%w+)!(.*)!(.*)!(.*)$")
	if treq and rn then
    	rv = {
    		 ifn  = ifn
    		,treq = treq
    		,rn   = rn
    		,da   = da
    		,ti   = ti
    		,wkId = wkId
    	}
    end
    return rv
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local queue = require"memcQueue":new("WKQueue", __set_wk_data, __get_wk_data) -- meta class

function WakeUpQueue:new()
	local obj = {}
	setmetatable(obj, {__index = queue})
	
	return obj
end

return WakeUpQueue
