--- astDb driver for raw access
-- 
-- @file	astDbRawInterface.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/23

--- @module astDbRawInterface
local astDbRawInterface = {
	_instance  = nil,
}

----------------------------------------------------------------
-- singleton implementation
----------------------------------------------------------------
function astDbRawInterface:getInstance(prefix)
    if self._instance then
    	return self._instance
    end

	self._instance  = astDbRawInterface
	local o         = self._instance
		  o.prefix  = prefix

	--- Execute native PbxTransaction service
	-- @param data	A data used for execution
	-- 		  cfg => {prefix=<prefix>}
	-- 		  req => {act="PUT/GET/DEL/DELTREE", key=<key>, val=<val>}
	-- @return boolean state or operation 
	o.request = function(cfg, req)
		if "table" ~= type(req) then
			return false, "Wrong data while calling 'astDB' application"
		end
		
		local prefix = o.prefix
		if "table" == type(cfg) then
			prefix = cfg.prefix or prefix
		end

		local dModel     = require"DataModel"
		local PBX	 = dModel.PBX
		
		local cont_type  = req.act
		if "PUT" == cont_type then
			cont_type = PBX.P_DB_PUT
		elseif "GET" == cont_type then
			cont_type = PBX.P_DB_GET
		elseif "DEL" == cont_type then
			cont_type = PBX.P_DB_DEL
		elseif "DELTREE" == cont_type then
			cont_type = PBX.P_DB_DELTREE
		else
			cont_type = PBX.P_INVALID_IDENT
		end
		
		local astDBPkt, err = dModel:new({ident = "PBX", ct = cont_type})
		if not astDBPkt then
			return false, err
		end
		
		astDBPkt:addRepeatedTuple("opts.items", "Family", prefix)
		if req.key then
			astDBPkt:addRepeatedTuple("opts.items", "Key", req.key)
		end
		if req.val then
			astDBPkt:addRepeatedTuple("opts.items", "Val", req.val)
		end
		
		local __handleDBValue = function(pkt)
			local rs  = false
        	local err = "Unknown retrieved data type!"
        	
        	if PBX.P_EVENT == pkt:getItem("ident") and 517 == pkt:getItem("data.sub") then
        		local items = pkt:getRepeatedItem("data")
        		if "table" == type(items) then
        			rs  = items["Val"]
        			err = false		
        		end
        	end
        	
        	return rs, err
        end
        	
		return require"appHelper":startTransactionWithHandleResponce(astDBPkt, PBX.P_DB_GET == cont_type and __handleDBValue, true)		
	end

    return o
end

return astDbRawInterface
