--- memcache driver for raw access
-- 
-- @file	memcRawInterface.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/05/23

-- XXX: make support for PUT method with a time value

--- @module memcRawInterface
local memcRawInterface = {
	_instance  = nil,
}

----------------------------------------------------------------
-- singleton implementation
----------------------------------------------------------------
function memcRawInterface:getInstance(cfg)
    if self._instance then
    	return self._instance
    end

	self._instance  = memcRawInterface
	local o         = self._instance
	if "table" == type(cfg) then
		o.host   = cfg.host
		o.port   = cfg.port
		o.prefix = cfg.prefix
	end

	--- Execute manager memcashe service
	-- @param data	A data used for execution
	-- 		  cfg => {host=<host>, port=<port>}
	-- 		  req => {act="PUT/GET/DEL", key=<key>, val=<val>}
	-- @return boolean state or received data
	o.request = function(cfg, req)
		if "table" ~= type(req) then
			return false, "Wrong pushed data for memc service"
		end
		
		local prefix = o.prefix
		local host = o.host
		local port = tonumber(o.port)
		if "table" == type(cfg) then
			prefix = cfg.prefix or prefix 
			host   = cfg.host or host
			port   = tonumber(cfg.port) or port
		end
		if not host or not port or not prefix then
			return false, "Invalid input parameters"
		end
		
		local dModel     = require"DataModel"
		local SYS	 	 = dModel.SYS
		local memcSrvPkt, err = dModel:new({ident = "SYS", ct = SYS.M_MEMCACHE_SERVICE})
		if not memcSrvPkt then
			return false, err
		end
		
		local action = req.act
		if "PUT" == action then
			action = SYS.M_SRV_ACT_PUT
		elseif "GET" == action then
			action = SYS.M_SRV_ACT_GET
		elseif "DEL" == action then
			action = SYS.M_SRV_ACT_DEL
		elseif "INCR" == action then
			action = SYS.M_SRV_ACT_INC
		elseif "DECR" == action then
			action = SYS.M_SRV_ACT_DEC
		else
			action = SYS.M_SRV_ACT_UNDEF
		end
		
		memcSrvPkt:setItem("ServiceDataObj.action", action)
		
		memcSrvPkt:addRepeatedTuple("ServiceDataObj.data.items", "HOST", host)
		memcSrvPkt:addRepeatedTuple("ServiceDataObj.data.items", "PORT", port)
		if req.key then
			memcSrvPkt:addRepeatedTuple("ServiceDataObj.data.items", "KEY", prefix.."/"..req.key)
		end
		if req.val then
			memcSrvPkt:addRepeatedTuple("ServiceDataObj.data.items", "VALUE", req.val)
		end
		
		local memcErr  = nil
		local rv, dsID = require"appHelper":startTransaction(memcSrvPkt, true)
		if rv and dsID then
			local memcSrvInfo, err = dModel:new(bt_data(dsID))
			if not memcSrvInfo then
				memcErr = err
			else
				local items = memcSrvInfo:getRepeatedItem("options")
				if items and "table" == type(items) then
					rv = items["VALUE"]
				else
					memcErr = "Unknown retrieved data type"
				end
			end
		end
		
		return rv, memcErr
	end

    return o
end

return memcRawInterface

