--- Hotel database accessor
-- 
-- @file	HotelAccessor.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/04/07

--- @module HotelAccessor
local HotelAccessor = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------
-- TODO: presist connection must be closed
local function __exec(driver, functor, key, val)
	local err  = nil
	local data = nil
	if "runtime" == driver.mode or not driver.connected then
		_, err = driver:connect()
	end

	if not err then
   		data, err = functor(driver, key, val)
   	
    	if "runtime" == driver.mode then
    		driver:disconnect()
    	end
	end

	return data, err
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local accessor = require"IAccessor":new()        -- meta class

--- @see description in IAccessor
function accessor:put(key, val)
	if "table" ~= type(key) or "table" ~= type(val) then
		return false
	end
	if "sql" ~= key.ident then
		return false
	end
	
	return __exec(self.driver, self.driver.put, key.val, val)
end

--- @see description in IAccessor
function accessor:get(key)
	if "table" ~= type(key) or "sql" ~= key.ident then
		return false
	end
	return __exec(self.driver, self.driver.get, key.val)
end

--- @see description in IAccessor
function accessor:del(key)
	if "table" ~= type(key) or "sql" ~= key.ident then
		return false
	end
	
	return __exec(self.driver, self.driver.del, key.val)
end


--- Constructor
function HotelAccessor:new(cfg)
	local err = nil
	local obj = {}
	setmetatable(obj, {__index = accessor})

	obj.driver, err = require"sqliteDriver":new(cfg)
	if not obj.driver then
		return false, err
	end

	return obj
end

return HotelAccessor
