--- interface for all data accessors
-- 
-- @file	IAccessor.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/04/06

--- @module IAccessor
local IAccessor = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local accessor = { 
	driver = require"IDriver":new()
}

--- Put data to a storage
-- @param key           data key
-- @param val           data value
-- @return boolean status of operation
-- @note both, key and value can be represented as a simple string or,
--       it can has a structure like:
--          key -> {ident="value",val="user_key_to_delete"}
--          val -> {ident="value",val="user_key_to_delete"}
--
--          key -> {ident="sql",val={act="new/upd",table="",fields={"","","",..},where=""}}
--          val -> defines
function accessor:put(key, val)
end

--- Get data from storage
-- @param key           data key
-- @return requested data otherwise FALSE value with error message.
-- 		   In case sql requests will be returned simple values or iterator function
-- @note the key can be represented as a simple string or,
--       it can has a structure like:
--          {ident="value",val="user_key_to_delete"}
--          {ident="sql",val={table="",where=""}}
function accessor:get(key)
end

--- Delete data from storage
-- @param key           data key
-- @note the key can be represented as a simple string or,
--       it can has a structure like:
--          {ident="tree",val="user_key_to_delete"}    --> 'delTree' handler
--          {ident="value",val="user_key_to_delete"}   --> 'del' handler
--          {ident="sql",val={table="",where=""}}      --> 'sql delete' handler
function accessor:del(key)
end


--- Object constructor
function IAccessor:new()
	local obj = {}
	setmetatable(obj, {__index = accessor})
	return obj
end

return IAccessor
