--- interface for all drivers
-- 
-- @file	IDriver.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/04/07

--- @module IDriver
local IDriver = {}

----------------------------------------------------------------
-- public interface
----------------------------------------------------------------
local driver = { 
	mode 	  = "runtime",
	connected = false
}

--- Put data to a storage
-- @param key           data key
-- @param val           data value
-- @return boolean status of operation
-- @note for sql-oriented storages format of 'key'/'val' should be
--          key -> {act="new/upd",table="",fields={"","","",..}}
--          val -> {values={"","","",..}}
function driver:put(key,val)
end

--- Execute retrieving of data from external storage
-- @param key           data key
-- @return status,value; in case "no errors" status should be 'true'
-- @note for sql-oriented storages format of 'key' should be
--          {table="",where="",postfix=""}
function driver:get(key)
end

--- Execute deleting of data
-- @param key           data key
-- @note for sql-oriented storages format of 'key' should be
--          {table="",where=""}
function driver:del(key)
end

--- Execute deleting of data tree
-- @param key           data key, string
-- @note only for 'key->value' storages in astDriver
function driver:delTree(key)
end

--- Connect to the data storage
function driver:connect()
	self.connected = true
end

---Connect to the data storage
function driver:disconnect()
	self.connected = false
end

--- Check connection status
function driver:getConnected()
	return self.connected
end


--- Constructor
function IDriver:new()
	local obj = {}
	setmetatable(obj, {__index = driver})
	return obj
end

return IDriver
