--- Memory storage accessor
-- 
-- @file	MemAccessor.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/04/07

--- @module MemAccessor
local MemAccessor = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------
-- TODO: presist connection must be closed
local function __exec(driver, functor, key, val)
	local rv  = nil
	local err = nil
	if "runtime" == driver.mode or not driver.connected then
		_, err = driver:connect()
	end

	if not err then
   		rv, err = functor(driver, key, val)
    
    	if "runtime" == driver.mode then
    		driver:disconnect()
    	end
    end
	
	return rv, err
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local accessor  = require"IAccessor":new()        -- meta class

--- @see description in IAccessor
function accessor:put(key,val)
	if not key or "" == key then
		return false
	end
	if not val or "" == val then
		return false
	end
	
	return __exec(self.driver, self.driver.put, key, val)
end

--- @see description in IAccessor
function accessor:get(key)
	if not key or "" == key then
		return false
	end
	
	return __exec(self.driver, self.driver.get, key)
end

--- @see description in IAccessor
function accessor:del(key)
	if not key or "" == key then
		return false
	end
	
	return __exec(self.driver, self.driver.del, key)
end


--- Object constructor
function MemAccessor:new(cfg)
	local obj = {}
	setmetatable(obj, {__index = accessor})

	if "table" == type(cfg) then
		local factory = require"DriversFactory":new(cfg.options)
		obj.driver = factory:instantiate(cfg.ident)
	end
	
	return obj
end

return MemAccessor
