--- astDb driver
-- 
-- @file	astDriver.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/04/07

--- @module astDriver
local astDriver = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local driver  = require"IDriver":new()        -- meta class
driver.prefix = nil		-- family in astDB requests

--- @see description in IDriver
function driver:put(key, val)
	return self.raw.request({prefix = self.prefix},{act = "PUT", key = key, val = val})
end

--- @see description in IDriver
function driver:get(key)
	return self.raw.request({prefix = self.prefix},{act = "GET", key = key})
end

--- @see description in IDriver
function driver:del(key)
	return self.raw.request({prefix = self.prefix},{act = "DEL", key = key})
end

--- @see description in IDriver
function driver:delTree(key)
	return self.raw.request({prefix = self.prefix},{act = "DELTREE", key = key})
end


--- Constructor
function astDriver:new(cfg)
	if "table" ~= type(cfg) then
		return false, "AstDB: configuration not found"
	end
	if not cfg.prefix or "" == cfg.prefix then
		return false, "AstDB: prefix variable not found"
	end
	
	local obj = {}
	setmetatable(obj, {__index = driver})
	
	obj.prefix = cfg.prefix
	obj.raw    = require("astDbRawInterface"):getInstance()
	
	return obj
end

return astDriver
