--- memcache simple driver
-- 
-- @file	memcSimpleDriver.lua 
-- @author	Pavel Yadvichuk
-- @created 2015/04/07

--- @module memcSimpleDriver
local memcSimpleDriver = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local driver = require"IDriver":new()        -- meta class
driver.cfg   = {
	host = nil,		-- memcached host data
	port = nil,		-- memcached port value
}

--- @see description in IDriver
function driver:put(key, val)
	return self.raw.put(key,val)
end

--- @see description in IDriver
function driver:get(key)
	return self.raw.get(key)
end

--- @see description in IDriver
function driver:del(key)
	return self.raw.del(key)
end

--- Connect to the data storage
function driver:connect()
	local err = nil
	if not self.connected then
		self.connected,err = self.raw.connect(self.cfg.host,self.cfg.port)
	end
	return self.connected, err
end

--- Disconnect from the data storage
function driver:disconnect()
	self.connected = self.raw.disconnect()
end


--- Constructor
function memcSimpleDriver:new(cfg)
	if "table" ~= type(cfg) then
		return false, "MemcSimple: configuration not found"
	end
	if not cfg.host or "" == cfg.host then
		return false, "MemcSimple: invalid host data in config"
	end
	if not tonumber(cfg.port) then
		return false, "MemcSimple: invalid port in config"
	end
	
	local obj = {}
	setmetatable(obj, {__index = driver})
	
	-- init configuration	
	obj.cfg = cfg
	obj.raw = require("memcSimpleRawInterface"):getInstance()

	return obj
end

return memcSimpleDriver
