--- Application helper
-- 
-- @file    appHelper.lua 
-- @author  Vyacheslav Khudan
-- @created 2015/05/20

--- @module appHelper
local appHelper = {}

--- Generate transaction for created packet
-- @param pkt		A sending packet
-- @param silence	A flag for disabling verbose logs
-- @return boolean value and data set ID if present
function appHelper:startTransaction(pkt, silence)
	if not pkt then
		return false
	end
	
	if not pkt:isSystem() then
		local statusController = require"ModStatusController":getInstance()
		local modName 			  = pkt:getTypeString()
		local modStatus 		  = statusController.getStatus(modName)
		if "online" ~= modStatus then
			print("Interface %q status is %q. Dropping packet ...", modName, modStatus)
			return false
		end
	end
	
	if not silence then	--TODO: use low level notification logs
		printdbg(tostring(pkt)) --TODO: remove using tostring function
	end
	
	local pktType      = pkt:getTypeString()
	local dataSet, err = pkt:serialize()
	if not dataSet then
		print("%s MSG data serialization error!", pktType)
		return false
	end
	
	local rs   = nil
	local dsID = nil
	if pkt:isXopen() then
		rs, dsID = app.XOpenTransaction(dataSet)
	elseif pkt:isFias() then
		rs, dsID = app.FiasTransaction(dataSet)
	elseif pkt:isPbx() then
		rs, dsID = app.PbxTransaction(dataSet)
	elseif pkt:isSystem() then
		rs, dsID = app.MgrService(dataSet)
	end
	
	if not silence then	--TODO: use low level notification logs
		printdbg("%s transaction result is %s %s", pktType, rs and "OK" or "FAIL", dsID and '('..dsID..')' or "")
	end
	
	return rs, dsID
end

--- Generate transaction for created packet with handling response
-- @param pkt		A sending packet
-- @param callback	A callback function for processing transaction response
-- @param silence	A flag for disabling verbose logs
-- @return boolean value or callback result
function appHelper:startTransactionWithHandleResponce(pkt, callback, silence)
	local errMsg   = "Unknown error"
	local rs, dsID = self:startTransaction(pkt, silence)
	
	if "function" ~= type(callback) then
		return rs, not rs and errMsg
	end
	
	repeat
		if not rs or not dsID then
			break
		end
		
		rs = false  -- reset result value
		local respData, err = require"DataModel":new(bt_data(dsID))
		if not respData then
			errMsg = err
			break
		end
		
		rs, errMsg = callback(respData)
	until true
	
	return rs, errMsg
end

return appHelper 