--- Codeset facility 
-- 
-- @file    codeSetConv.lua 
-- @author  Vyacheslav Khudan
-- @created 2015/08/12

--@module codeSetConv
local codeSet = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local codeset = {	-- meta Class
	toCodeset   = nil,	-- destination caracters codeset
	fromCodeset = nil,	-- source caracters codeset
}

--- Converts the sequence of characters from one codeset to another one
-- @param str	An input string for conversion
-- @return converted string otherwise FALSE with error message 
function codeset:conv(str)
	if "string" ~= type(str) or "" == str then
		return false, "Invalid input string format"
	end
	
	if not self.toCodeset or not self.fromCodeset then
		return false, "Source and destination codesets(s) does not specified"
	end
	
	if self.toCodeset == self.fromCodeset then
		return str
	end
	
	local iconv = require"iconv"
	local cd = iconv.new(self.toCodeset, self.fromCodeset)
	if not cd then
		return false, string.format("Uses invalid codeset %q:%q", self.fromCodeset, self.toCodeset)
	end
	
	local errMsg = { -- string representation for iconv error messages
		 [iconv.ERROR_NO_MEMORY] 	= "Failed to allocate enough memory in the conversion process"
		,[iconv.ERROR_INVALID]   	= "An invalid character was found in the input sequence"
		,[iconv.ERROR_INCOMPLETE] 	= "An incomplete character was found in the input sequence"
--		,[iconv.ERROR_FINALIZED] 	= "Trying to use an already-finalized converter" -- present in 'iconv r7'
		,[iconv.ERROR_UNKNOWN]	 	= "There was an unknown error"
	}
	
	local rv, errCode = cd:iconv(str)
	
	return rv, errCode and errMsg[errCode]	
end


--- Object constructor
function codeSet:new(fromCodeset, toCodeset)
	local obj = {}
	setmetatable(obj, {__index = codeset})
	
	-- initialization
	obj.toCodeset 	 = toCodeset or "//TRANSLIT"
	obj.fromCodeset = fromCodeset
	
	return obj
end

return codeSet