--- Convertor for string to/from HEX data
-- 
-- @file    strConverter.lua 
-- @author  Vyacheslav Khudan
-- @created 2015/06/01

--- @module strConverter
local strConverter = {}

--- Convert string value to HEX
-- @param str   A string value
-- @return HEX representation of the string
function strConverter.toHex(str)
    if "string" ~= type(str) or "" == str then
        return ""
    end
    
    local rv = ""
    for i = 1, #str do
        rv = rv .. string.format("%02x", str:byte(i))
    end
    
    return rv
end

--- Convert HEX to string
-- @param	An input HEX data
-- @return string representation of the data
function strConverter.fromHex(hex)
	if not hex or "" == hex then
		return ""
	end
	
	local val = nil
	hex = tostring(hex)
    
    local charCodes = {}	
	while hex and #hex > 0 do
		val, hex = hex:match("(..)(.*)$")
		if val then
		    local b = string.format("%d", tonumber(val, 16))  
			table.insert(charCodes, b)
		end
	end
	
	return string.char(unpack(charCodes))  
end

--- Encode string to base64 format
-- @param str	A string value
-- @return binary data
function strConverter.toB64(str)
	if "string" ~= type(str) or "" == str then
		return ""
	end
	
	return require"base64".encode(str)
end

--- decode binary data from base64 format
-- @param data	An input binary data
-- @return string representation of the data
function strConverter.fromB64(data)
	if not data then
		return ""
	end
	
	return require"base64".decode(data)
end

return strConverter