--- Module to manage voicemail messages by extension
-- 
-- @file    userVoicemail.lua 
-- @author  Vyacheslav Khudan
-- @created 2018/10/05

-- load additional modules
local modPathList = {
     "/usr/local/share/lua/5.1/common/?.lua"
}
package.path  = package.path .. ";" .. table.concat(modPathList, ";")

--- @module userVoicemail
local userVoicemail = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------
--- Quiet module loading
local function __requireQuiet(modName)
    if not modName then
        return false
    end

    for _, modPath in ipairs(modPathList) do
        if modPath then
            local modFile = string.gsub(modPath, "?", modName)
            local file    = io.open(modFile)
            if file then
                io.close(file)
                return require(modName)
            end
        end
    end
    
    return false, "User voicemail module file not found in controllers directory"
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
--- Remove all voicemail messages for room
-- @param exten     A room identificator at PBX
-- @return number of the removed voicemails otherwise FALSE with error message
function userVoicemail.removeAll(exten)
    local VMApp, err = __requireQuiet("voiceMailApplication")
    if not VMApp then
        return false, err
    end
    
    if not tonumber(exten) then
        return false, "Invalid extension id"
    end
    
    local rv      = 0
    local userVoicemailList, err = VMApp.getMsgInfo({mailbox = exten})
    if "table" ~= type(userVoicemailList) then
        return rv
    end
    
    for idx in ipairs(userVoicemailList) do
        if VMApp.deleteMsg({mailbox = exten, msgId = userVoicemailList[idx].msgId}) then
            rv = rv + 1
        end
    end
    
    if 0 < rv then
        VMApp.updateVM(exten)
    end
    return rv
end

return userVoicemail